;;; df.el --- Hack for displaying space left on device in the mode line

;; Copyright (C) 1999 by Association April

;; Author: Benjamin Drieu <drieu@bocal.cs.univ-paris8.fr>
;; Keywords: unix, tools

;; This file is NOT part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;;  This is a slight hack for displaying the disk usage in the mode
;;  line.  Disk space is updated every df-refresh second.

;;  If you work with numerous users sharing the same partition, it
;;  happens that you haven't place left to save your work, which
;;  drives you to serious brain damage when you lost important work.
;;  This package allows you to have place left and buffer size
;;  displayed in the mode line, so you know when you can save your
;;  file.

;;  This package may (must) not be very optimized or efficient, but
;;  this is a quick hack.  Comments and suggestions are welcome.

;;  df is simple to use :
;;  - Put this file in your load-path
;;  - Autoload it : (autoload 'df "df" nil t)
;;  - Add something like (df "/home") in your .emacs if you want to
;;    scan /home

;; $Id: df.el,v 1.5 1999/01/24 17:25:54 drieu Exp $

;; $Log: df.el,v $
;; Revision 1.5  1999/01/24 17:25:54  drieu
;; - Add Paal Steihaug remarks :
;;   + use magic df argument, which only scan a partition
;;   + add (require 'cl)
;;   + df-update is now much clean
;;   + df now use either 'df -m' or 'df -k' when it is needed
;;
;; Revision 1.4  1999/01/04 14:51:01  drieu
;; - Correct a bug so Megabytes are *REALLY* Megabytes
;;
;; Revision 1.3  1999/01/02 15:46:44  drieu
;; - Fix few bugs one more time
;; - Add variables instead of hard-coded strings
;; - Add argument for df
;; - Document the file a bit more
;;
;; Revision 1.2  1998/12/15 17:37:42  drieu
;; - Fix few bugs
;; - Add Buffer size in the mode line
;; - Mesure either in K or Mega bytes
;; - And so on...



;;; Code:

;; You need it because of 'when'
(require 'cl)

;; Variables that you may want to change
(defvar df-partition "/home" "*Partition to scan")
(defvar df-command "df" "*df command in order to find how to get disk usage")
(defvar df-in-kilobytes "-k" "*Argument to use when df works in kilobytes")
(defvar df-in-megabytes "-m" "*Argument to use when df works in megabytes")
(defvar df-command-arguments df-in-kilobytes "*df-command arguments")
(defvar df-refresh 3 "*Seconds between every refresh")
(defvar df-megabytes-unit "M" "String used for displaying megabytes")
(defvar df-kilobytes-unit "K" "String used for displaying kilobytes")

;; Seemless variables for you
(defvar df-mb-threshold 10 "*When free disk space reaches this amount (in Mb), show in Mb")
(defvar df-space-left "" "Space left on device")
(defvar df-unit nil "Unit (either M or K) used for space left")


(defun df-change-mode-line-format (l)
  "Add the space left at the end of each mode line"
  (cond
   ;; Because sometimes mode line is a string
   ((stringp l) l)
   ((null (cdr l))
    (append (list "-" 'df-buffer-weight 'df-kilobytes-unit "/" 'df-space-left 'df-unit) l))
   ((append (list (car l)) (df-change-mode-line-format (cdr l))))))



(defun df-update ()
  "*Function for updating disk usage"
  (interactive)
  (set-variable 'df-buffer-weight (int-to-string (/ (length (buffer-string)) 1000)))
  (cond
   ((> (string-to-int df-space-left) (* df-mb-threshold 1000))
    (set-variable 'df-unit df-megabytes-unit)
    (setq df-command-arguments "-m"))
   ((and (< (string-to-int df-space-left) df-mb-threshold)(string-equal df-command-arguments "-m"))
    (set-variable 'df-unit df-kilobytes-unit)
    (setq df-command-arguments "-k"))
   ((not df-unit)
    (set-variable 'df-unit df-kilobytes-unit)))
  (set-process-filter (start-process "df" nil df-command df-command-arguments df-partition) 'df-filter))


(defun df-filter (proc string)
  "Filter for df output"
  (when (string-match (format "\\(-?[0-9]+\\) *[0-9%%]+ *%s" df-partition) string)
    (setq df-space-left (match-string 1 string))
    (if (> (string-to-int df-space-left) 1000)
	(set-variable 'df-unit df-megabytes-unit)
      (set-variable 'df-unit df-kilobytes-unit))
    (when (equal df-unit df-megabytes-unit) 
      (setq df-space-left (substring df-space-left 0 (- (length df-space-left) 3))))))



(defun df-change-all-mode-lines ()
  "Change all the mode line for displaying the track number"
  (interactive)
  (set-variable 'default-mode-line-format (df-change-mode-line-format default-mode-line-format))
  (df-change-one-mode-line (buffer-list)))



(defun df-change-one-mode-line (L)
  "Change the mode line of one buffer"
  (if (null L)
      nil
    (set-buffer (get-buffer (car L)))
    (setq mode-line-format (df-change-mode-line-format mode-line-format))
    (force-mode-line-update)
    (df-change-one-mode-line (cdr L))))


(defun df-stop ()
  "*Stops all df actions, but do not *yet* restore mode lines"
  (interactive)
  (cancel-function-timers 'df-update))


(defun df (&optional partition)
  "*Function for displaying disk statistics in the mode line"
  (interactive)
  (when partition
    (set-variable 'df-partition partition))
  (make-local-variable 'df-buffer-weight)
  (run-with-timer 0 df-refresh 'df-update)
  (df-change-all-mode-lines))

;;; df.el ends here